==> spacejunk-1.0.3.tar.gz: spacejunk-1.0.3/src/simulation.cpp <==
/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "simulation.h"
#include "collisionengine.h"
#include "functionexts.h"

using namespace std;

Simulation::Simulation(const PlayerState & ps,const list<int> &onStep)
        : physic(ps),tevents(this),onStep(onStep),colPrecision(35),deltaCol(0) {
    ce=new CollisionEngine<PhysicBody,Simulation>::CollisionEngine(-20000,-20000,20000,20000,this);
    for (vector<PhysicBody*>::iterator i=physic.bodies.begin();
            i!=physic.bodies.end();i++)
        if (*i) ce->add(*i);
};


class Collision : public Event {
public:
    Simulation * sim;
    int c1,c2;
    Vector2d normal;

    Collision() : sim(NULL),c1(-1),c2(-1),normal(0) {};
    Collision(Simulation * sim,int c1,int c2,const Vector2d & normal):
            sim(sim),c1(c1),c2(c2),normal(normal) {};
    void setQueue(Simulation * sim,const TimerId & tid) {}
    int fire(Simulation * sim);
    void cancel(Simulation * sim) {}
};


int Collision::fire(Simulation * sim) {
    PhysicBody * b1=sim->physic.bodies[c1];
    PhysicBody * b2=sim->physic.bodies[c2];
    if (!b1 || !b2) return 0;

    b1->testForCollision();
    b2->testForCollision();

    for (EventListenerList<SimCollisionListener*>::iterator i=sim->collisionListeners.begin();i!=sim->collisionListeners.end();i++)
        (*i)->evtCollision(sim,b1,b2,normal);
    return 0;
}


int handle_col(Simulation * sim,PhysicBody * c1,PhysicBody * c2,real t,const Vector2d & normal) {
    int key=(int)(t*sim->physic.getInterpolatedTime());
    sim->coldata.push_front(make_pair(key,Collision(sim,c1->id,c2->id,normal)));
    return 0;
};

int Simulation::handle_collisions() {
    ce->step(wrap_collision_handler<PhysicBody,Simulation,handle_col>);
    if (!coldata.empty()) {
        int t=physic.unstep();
        tevents.clock-=t;
        for (list<std::pair<int,Collision> >::iterator i=coldata.begin();coldata.end()!=i;i++) {
            tevents.set(i->first,i->second);
        }
        coldata.clear();
        return t;
    }
    return 0;
};

void Simulation::step(int delta) {
    step(delta,null);
};

void Simulation::clear() {
    for (vector<PhysicBody*>::iterator i=physic.bodies.begin();i!=physic.bodies.end();i++) {
        if (!*i) continue;
        deleteBody((*i)->id);
    }
    physic.clear();
};

void Simulation::purge() {
    for (vector<PhysicBody*>::iterator i=physic.bodies.begin();i!=physic.bodies.end();i++) {
        if (!*i) continue;
        PhysicBody * pb=*i;
        deleteBody_without_event((*i)->id);
        delete pb;
    }
    physic.clear();
    tevents.clear();
};

void Simulation::addBody(PhysicBody * c) {
    physic.addBody(c);
    ce->add(c);
}

void Simulation::addBody(int id,PhysicBody * c) {
    physic.addBody(id,c);
    ce->add(c);
}

void Simulation::deleteBody(int id) {
    if (id<0 || !physic.bodies[id]) return;

    for (EventListenerList<SimDeletionListener*>::iterator i=deletionListeners.begin();i!=deletionListeners.end();i++)
        (*i)->evtDeleteBody(this,id);

    deleteBody_without_event(id);
}

void Simulation::deleteBody_without_event(int id) {
    ce->deleteBody(physic.bodies[id]);
    physic.deleteBody(id);
}

Simulation::~Simulation() {
    delete ce;
};

void Simulation::operator = (const Simulation & orig) {
    purge();

    physic=orig.physic;
    for (vector<PhysicBody*>::const_iterator i=physic.bodies.begin();i!=physic.bodies.end();i++) {
        if (!*i) continue;
        ce->add(*i);
    }

    colPrecision=orig.colPrecision;
    deltaCol=orig.deltaCol;
    orig.tevents.copy(tevents);
};

==> spacejunk-1.0.5.tar.gz: spacejunk-1.0.5/src/simulation.cpp <==
/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "simulation.h"
#include "collisionengine.h"
#include "functionexts.h"

using namespace std;

Simulation::Simulation(const PlayerState & ps,const list<int> &onStep)
        : physic(ps),tevents(this),onStep(onStep),colPrecision(35),deltaCol(0) {
    ce=new CollisionEngine<PhysicBody,Simulation>(-20000,-20000,20000,20000,this);
    for (vector<PhysicBody*>::iterator i=physic.bodies.begin();
            i!=physic.bodies.end();i++)
        if (*i) ce->add(*i);
};


class Collision : public Event {
public:
    Simulation * sim;
    int c1,c2;
    Vector2d normal;

    Collision() : sim(NULL),c1(-1),c2(-1),normal(0) {};
    Collision(Simulation * sim,int c1,int c2,const Vector2d & normal):
            sim(sim),c1(c1),c2(c2),normal(normal) {};
    void setQueue(Simulation * sim,const TimerId & tid) {}
    int fire(Simulation * sim);
    void cancel(Simulation * sim) {}
};


int Collision::fire(Simulation * sim) {
    PhysicBody * b1=sim->physic.bodies[c1];
    PhysicBody * b2=sim->physic.bodies[c2];
    if (!b1 || !b2) return 0;

    b1->testForCollision();
    b2->testForCollision();

    for (EventListenerList<SimCollisionListener*>::iterator i=sim->collisionListeners.begin();i!=sim->collisionListeners.end();i++)
        (*i)->evtCollision(sim,b1,b2,normal);
    return 0;
}


int handle_col(Simulation * sim,PhysicBody * c1,PhysicBody * c2,real t,const Vector2d & normal) {
    int key=(int)(t*sim->physic.getInterpolatedTime());
    sim->coldata.push_front(make_pair(key,Collision(sim,c1->id,c2->id,normal)));
    return 0;
};

int Simulation::handle_collisions() {
    ce->step(wrap_collision_handler<PhysicBody,Simulation,handle_col>);
    if (!coldata.empty()) {
        int t=physic.unstep();
        tevents.clock-=t;
        for (list<std::pair<int,Collision> >::iterator i=coldata.begin();coldata.end()!=i;i++) {
            tevents.set(i->first,i->second);
        }
        coldata.clear();
        return t;
    }
    return 0;
};

void Simulation::step(int delta) {
    step(delta,null);
};

void Simulation::clear() {
    for (vector<PhysicBody*>::iterator i=physic.bodies.begin();i!=physic.bodies.end();i++) {
        if (!*i) continue;
        deleteBody((*i)->id);
    }
    physic.clear();
};

void Simulation::purge() {
    for (vector<PhysicBody*>::iterator i=physic.bodies.begin();i!=physic.bodies.end();i++) {
        if (!*i) continue;
        PhysicBody * pb=*i;
        deleteBody_without_event((*i)->id);
        delete pb;
    }
    physic.clear();
    tevents.clear();
};

void Simulation::addBody(PhysicBody * c) {
    physic.addBody(c);
    ce->add(c);
}

void Simulation::addBody(int id,PhysicBody * c) {
    physic.addBody(id,c);
    ce->add(c);
}

void Simulation::deleteBody(int id) {
    if (id<0 || !physic.bodies[id]) return;

    for (EventListenerList<SimDeletionListener*>::iterator i=deletionListeners.begin();i!=deletionListeners.end();i++)
        (*i)->evtDeleteBody(this,id);

    deleteBody_without_event(id);
}

void Simulation::deleteBody_without_event(int id) {
    ce->deleteBody(physic.bodies[id]);
    physic.deleteBody(id);
}

Simulation::~Simulation() {
    delete ce;
};

void Simulation::operator = (const Simulation & orig) {
    purge();

    physic=orig.physic;
    for (vector<PhysicBody*>::const_iterator i=physic.bodies.begin();i!=physic.bodies.end();i++) {
        if (!*i) continue;
        ce->add(*i);
    }

    colPrecision=orig.colPrecision;
    deltaCol=orig.deltaCol;
    orig.tevents.copy(tevents);
};
